#include "ELV-BM-MCU.h"

unsigned int rotation_counter = 0;
const unsigned int measure_period_seconds = 10; 

#define SEVEN_SEG_PIN_A1 PIN07
#define SEVEN_SEG_PIN_B1 PIN03
#define SEVEN_SEG_PIN_C1 PIN02
#define SEVEN_SEG_PIN_D1 PIN01

#define SEVEN_SEG_PIN_A2 PIN21
#define SEVEN_SEG_PIN_B2 PIN22
#define SEVEN_SEG_PIN_C2 PIN23
#define SEVEN_SEG_PIN_D2 PIN24

#define PIN_ELV_AM_Hall_LATCH PIN04

unsigned int calculateRotationsPerMinute()
{
  unsigned int rpm = (unsigned int)((float)rotation_counter / (float)measure_period_seconds * 60.0f);
  rotation_counter = 0;
  return rpm;
}

void ISR1_anemometer_callback()
{
  rotation_counter++;
}

void setup() {
  pinMode(SEVEN_SEG_PIN_A1, OUTPUT);
  pinMode(SEVEN_SEG_PIN_B1, OUTPUT);
  pinMode(SEVEN_SEG_PIN_C1, OUTPUT);
  pinMode(SEVEN_SEG_PIN_D1, OUTPUT);

  pinMode(SEVEN_SEG_PIN_A2, OUTPUT);
  pinMode(SEVEN_SEG_PIN_B2, OUTPUT);
  pinMode(SEVEN_SEG_PIN_C2, OUTPUT);
  pinMode(SEVEN_SEG_PIN_D2, OUTPUT);

  pinMode(PIN_ELV_AM_Hall_LATCH, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(PIN_ELV_AM_Hall_LATCH), ISR1_anemometer_callback, RISING);

  digitalWrite(SEVEN_SEG_PIN_A1, LOW);
  digitalWrite(SEVEN_SEG_PIN_B1, LOW);
  digitalWrite(SEVEN_SEG_PIN_C1, LOW);
  digitalWrite(SEVEN_SEG_PIN_D1, LOW);

  digitalWrite(SEVEN_SEG_PIN_A2, LOW);
  digitalWrite(SEVEN_SEG_PIN_B2, LOW);
  digitalWrite(SEVEN_SEG_PIN_C2, LOW);
  digitalWrite(SEVEN_SEG_PIN_D2, LOW);
  Serial1.println("setup()");
}

void setOutputToDigit(unsigned int decimal, unsigned int output_number)
{
  if((0 <= decimal) && (decimal < 10))
  {
    if(output_number == 1)
    {
      digitalWrite(SEVEN_SEG_PIN_A1, bitRead(decimal, 0));
      digitalWrite(SEVEN_SEG_PIN_B1, bitRead(decimal, 1));
      digitalWrite(SEVEN_SEG_PIN_C1, bitRead(decimal, 2));
      digitalWrite(SEVEN_SEG_PIN_D1, bitRead(decimal, 3));
    }
    else if(output_number == 2)
    {
      digitalWrite(SEVEN_SEG_PIN_A2, bitRead(decimal, 0));
      digitalWrite(SEVEN_SEG_PIN_B2, bitRead(decimal, 1));
      digitalWrite(SEVEN_SEG_PIN_C2, bitRead(decimal, 2));
      digitalWrite(SEVEN_SEG_PIN_D2, bitRead(decimal, 3));
    }
  }
}

void loop() {
  delay(measure_period_seconds * 1000);
  noInterrupts();
  unsigned int rotations_per_seconds = calculateRotationsPerMinute();
  interrupts();
  if (rotations_per_seconds < 99)
  {
    setOutputToDigit(rotations_per_seconds%10, 1);
    setOutputToDigit((rotations_per_seconds/10)%10, 2);
  }

}
